@@ -39,6 +39,7 @@
 #include "cell.h"
 #include "debug.h"
 #include "delaunay.h"
+#include "determination.h"
 #include "kgrid.h"
 #include "kpoint.h"
 #include "mathfunc.h"
@@ -53,9 +54,6 @@
 #include "symmetry.h"
 #include "version.h"
 
-#define REDUCE_RATE 0.9
-#define NUM_ATTEMPT 10
-
 /*-------*/
 /* error */
 /*-------*/
@@ -94,7 +92,7 @@ static int set_dataset(SpglibDataset * dataset,
                        const Cell * cell,
                        const Primitive * primitive,
                        SPGCONST Spacegroup * spacegroup,
-                       const double tolerance);
+                       ExactStructure *exstr);
 static int get_symmetry_from_dataset(int rotation[][3][3],
                                      double translation[][3],
                                      const int max_size,
@@ -446,10 +444,9 @@ int spg_get_hall_number_from_symmetry(SPGCONST int rotation[][3][3],
                                       const int num_operations,
                                       const double symprec)
 {
-  int i;
+  int i, hall_number;
   Symmetry *symmetry;
   Symmetry *prim_symmetry;
-  Spacegroup spacegroup;
 
   symmetry = NULL;
   prim_symmetry = NULL;
@@ -461,11 +458,11 @@ int spg_get_hall_number_from_symmetry(SPGCONST int rotation[][3][3],
   }
 
   prim_symmetry = prm_get_primitive_symmetry(symmetry, symprec);
-  spacegroup = spa_search_spacegroup_with_symmetry(prim_symmetry, symprec);
+  hall_number = spa_search_spacegroup_with_symmetry(prim_symmetry, symprec);
 
-  if (spacegroup.hall_number) {
+  if (hall_number) {
     spglib_error_code = SPGLIB_SUCCESS;
-    return spacegroup.hall_number;
+    return hall_number;
   } else {
     spglib_error_code = SPGERR_SPACEGROUP_SEARCH_FAILED;
     return 0;
@@ -1016,17 +1013,13 @@ static SpglibDataset * get_dataset(SPGCONST double lattice[3][3],
                                    const double symprec,
                                    const double angle_tolerance)
 {
-  int attempt;
-  double tolerance;
-  Spacegroup spacegroup;
   SpglibDataset *dataset;
   Cell *cell;
-  Primitive *primitive;
+  DataContainer *container;
 
-  spacegroup.number = 0;
   dataset = NULL;
   cell = NULL;
-  primitive = NULL;
+  container = NULL;
 
   if ((dataset = init_dataset()) == NULL) {
     goto not_found;
@@ -1047,35 +1040,24 @@ static SpglibDataset * get_dataset(SPGCONST double lattice[3][3],
     goto atoms_too_close;
   }
 
-  tolerance = symprec;
-  for (attempt = 0; attempt < NUM_ATTEMPT; attempt++) {
-    if ((primitive = spa_get_spacegroup(&spacegroup,
-                                        cell,
-                                        hall_number,
-                                        tolerance,
-                                        angle_tolerance))
-        == NULL) {
+  if ((container = det_determine_all(cell,
+                                     hall_number,
+                                     symprec,
+                                     angle_tolerance))
+      != NULL) {
+    if (set_dataset(dataset,
+                    cell,
+                    container->primitive,
+                    container->spacegroup,
+                    container->exact_structure)) {
+      det_free_container(container);
+      container = NULL;
       cel_free_cell(cell);
       cell = NULL;
-      free(dataset);
-      dataset = NULL;
-      goto not_found;
+      goto found;
     }
-
-    if (spacegroup.number > 0) {
-      if (set_dataset(dataset,
-                      cell,
-                      primitive,
-                      &spacegroup,
-                      primitive->tolerance)) {
-        goto found;
-      } else {
-        tolerance *= REDUCE_RATE;
-      }
-    }
-
-    prm_free_primitive(primitive);
-    primitive = NULL;
+    det_free_container(container);
+    container = NULL;
   }
 
   cel_free_cell(cell);
@@ -1092,10 +1074,6 @@ static SpglibDataset * get_dataset(SPGCONST double lattice[3][3],
   return NULL;
 
  found:
-  prm_free_primitive(primitive);
-  primitive = NULL;
-  cel_free_cell(cell);
-  cell = NULL;
 
   spglib_error_code = SPGLIB_SUCCESS;
   return dataset;
@@ -1103,6 +1081,7 @@ static SpglibDataset * get_dataset(SPGCONST double lattice[3][3],
 
 static SpglibDataset * init_dataset(void)
 {
+  int i, j;
   SpglibDataset *dataset;
 
   dataset = NULL;
@@ -1130,6 +1109,11 @@ static SpglibDataset * init_dataset(void)
   dataset->n_std_atoms = 0;
   dataset->std_positions = NULL;
   dataset->std_types = NULL;
+  for (i = 0; i < 3; i++) {
+    for (j = 0; j < 3; j++) {
+      dataset->std_rotation_matrix[i][j] = 0;
+    }
+  }
   dataset->std_mapping_to_primitive = NULL;
   /* dataset->pointgroup_number = 0; */
   strcpy(dataset->pointgroup_symbol, "");
@@ -1142,58 +1126,12 @@ static int set_dataset(SpglibDataset * dataset,
                        const Cell * cell,
                        const Primitive * primitive,
                        SPGCONST Spacegroup * spacegroup,
-                       const double tolerance)
+                       ExactStructure *exstr)
 {
   int i;
-  int *std_mapping_to_primitive, *wyckoffs, *equivalent_atoms;
   double inv_lat[3][3];
-  Cell *bravais;
-  Symmetry *symmetry;
   Pointgroup pointgroup;
 
-  wyckoffs = NULL;
-  equivalent_atoms = NULL;
-  std_mapping_to_primitive = NULL;
-  bravais = NULL;
-  symmetry = NULL;
-
-  if ((symmetry = ref_get_refined_symmetry_operations(cell,
-                                                      primitive->cell,
-                                                      spacegroup,
-                                                      tolerance)) == NULL) {
-    return 0;
-  }
-
-  if ((wyckoffs = (int*) malloc(sizeof(int) * cell->size)) == NULL) {
-    warning_print("spglib: Memory could not be allocated.");
-    goto err;
-  }
-
-  if ((equivalent_atoms = (int*) malloc(sizeof(int) * cell->size)) == NULL) {
-    warning_print("spglib: Memory could not be allocated.");
-    goto err;
-  }
-
-  if ((std_mapping_to_primitive =
-       (int*) malloc(sizeof(int) * primitive->cell->size * 4)) == NULL) {
-    warning_print("spglib: Memory could not be allocated.");
-    goto err;
-  }
-
-  if ((bravais = ref_get_Wyckoff_positions(wyckoffs,
-                                           equivalent_atoms,
-                                           std_mapping_to_primitive,
-                                           primitive->cell,
-                                           cell,
-                                           spacegroup,
-                                           symmetry,
-                                           primitive->mapping_table,
-                                           tolerance)) == NULL) {
-    warning_print("spglib: ref_get_Wyckoff_positions failed.");
-    warning_print(" (line %d, %s).\n", __LINE__, __FILE__);
-    goto err;
-  }
-
   /* Spacegroup type, transformation matrix, origin shift */
   dataset->n_atoms = cell->size;
   dataset->spacegroup_number = spacegroup->number;
@@ -1206,7 +1144,7 @@ static int set_dataset(SpglibDataset * dataset,
                          inv_lat, cell->lattice);
   mat_copy_vector_d3(dataset->origin_shift, spacegroup->origin_shift);
 
-  dataset->n_operations = symmetry->size;
+  dataset->n_operations = exstr->symmetry->size;
 
   if ((dataset->rotations =
        (int (*)[3][3]) malloc(sizeof(int[3][3]) * dataset->n_operations))
@@ -1222,14 +1160,11 @@ static int set_dataset(SpglibDataset * dataset,
     goto err;
   }
 
-  for (i = 0; i < symmetry->size; i++) {
-    mat_copy_matrix_i3(dataset->rotations[i], symmetry->rot[i]);
-    mat_copy_vector_d3(dataset->translations[i], symmetry->trans[i]);
+  for (i = 0; i < exstr->symmetry->size; i++) {
+    mat_copy_matrix_i3(dataset->rotations[i], exstr->symmetry->rot[i]);
+    mat_copy_vector_d3(dataset->translations[i], exstr->symmetry->trans[i]);
   }
 
-  sym_free_symmetry(symmetry);
-  symmetry = NULL;
-
   /* Wyckoff positions */
   if ((dataset->wyckoffs = (int*) malloc(sizeof(int) * dataset->n_atoms))
       == NULL) {
@@ -1244,15 +1179,10 @@ static int set_dataset(SpglibDataset * dataset,
   }
 
   for (i = 0; i < dataset->n_atoms; i++) {
-    dataset->wyckoffs[i] = wyckoffs[i];
-    dataset->equivalent_atoms[i] = equivalent_atoms[i];
+    dataset->wyckoffs[i] = exstr->wyckoffs[i];
+    dataset->equivalent_atoms[i] = exstr->equivalent_atoms[i];
   }
 
-  free(wyckoffs);
-  wyckoffs = NULL;
-  free(equivalent_atoms);
-  equivalent_atoms = NULL;
-
   if ((dataset->mapping_to_primitive =
        (int*) malloc(sizeof(int) * dataset->n_atoms)) == NULL) {
     warning_print("spglib: Memory could not be allocated.");
@@ -1261,14 +1191,14 @@ static int set_dataset(SpglibDataset * dataset,
 
   debug_print("Refined cell after ref_get_Wyckoff_positions\n");
   debug_print(" (line %d, %s).\n", __LINE__, __FILE__);
-  debug_print_matrix_d3(bravais->lattice);
+  debug_print_matrix_d3(exstr->bravais->lattice);
 #ifdef SPGDEBUG
-  for (i = 0; i < bravais->size; i++) {
+  for (i = 0; i < exstr->bravais->size; i++) {
     printf("%d: %f %f %f\n",
-           bravais->types[i],
-           bravais->position[i][0],
-           bravais->position[i][1],
-           bravais->position[i][2]);
+           exstr->bravais->types[i],
+           exstr->bravais->position[i][0],
+           exstr->bravais->position[i][1],
+           exstr->bravais->position[i][2]);
   }
 #endif
 
@@ -1276,8 +1206,8 @@ static int set_dataset(SpglibDataset * dataset,
     dataset->mapping_to_primitive[i] = primitive->mapping_table[i];
   }
 
-  dataset->n_std_atoms = bravais->size;
-  mat_copy_matrix_d3(dataset->std_lattice, bravais->lattice);
+  dataset->n_std_atoms = exstr->bravais->size;
+  mat_copy_matrix_d3(dataset->std_lattice, exstr->bravais->lattice);
 
   if ((dataset->std_positions =
        (double (*)[3]) malloc(sizeof(double[3]) * dataset->n_std_atoms))
@@ -1299,15 +1229,11 @@ static int set_dataset(SpglibDataset * dataset,
   }
 
   for (i = 0; i < dataset->n_std_atoms; i++) {
-    mat_copy_vector_d3(dataset->std_positions[i], bravais->position[i]);
-    dataset->std_types[i] = bravais->types[i];
-    dataset->std_mapping_to_primitive[i] = std_mapping_to_primitive[i];
+    mat_copy_vector_d3(dataset->std_positions[i], exstr->bravais->position[i]);
+    dataset->std_types[i] = exstr->bravais->types[i];
+    dataset->std_mapping_to_primitive[i] = exstr->std_mapping_to_primitive[i];
   }
-
-  free(std_mapping_to_primitive);
-  std_mapping_to_primitive = NULL;
-  cel_free_cell(bravais);
-  bravais = NULL;
+  mat_copy_matrix_d3(dataset->std_rotation_matrix, exstr->rotation);
 
   /* dataset->pointgroup_number = spacegroup->pointgroup_number; */
   pointgroup = ptg_get_pointgroup(spacegroup->pointgroup_number);
@@ -1316,30 +1242,10 @@ static int set_dataset(SpglibDataset * dataset,
   return 1;
 
  err:
-  if (wyckoffs != NULL) {
-    free(wyckoffs);
-    wyckoffs = NULL;
-  }
-  if (equivalent_atoms != NULL) {
-    free(equivalent_atoms);
-    equivalent_atoms = NULL;
-  }
   if (dataset->std_positions != NULL) {
     free(dataset->std_positions);
     dataset->std_positions = NULL;
   }
-  if (std_mapping_to_primitive != NULL) {
-    free(std_mapping_to_primitive);
-    std_mapping_to_primitive = NULL;
-  }
-  if (bravais != NULL) {
-    cel_free_cell(bravais);
-    bravais = NULL;
-  }
-  if (symmetry != NULL) {
-    sym_free_symmetry(symmetry);
-    symmetry = NULL;
-  }
   if (dataset->std_mapping_to_primitive != NULL) {
     free(dataset->std_mapping_to_primitive);
     dataset->std_mapping_to_primitive = NULL;
@@ -1450,6 +1356,9 @@ static int get_symmetry_with_collinear_spin(int rotation[][3][3],
 
   cel_set_cell(cell, lattice, position, types);
 
+#ifdef SPGTEST
+  sym_nonspin = sym_get_operation(cell, symprec, angle_tolerance);
+#else
   if ((dataset = get_dataset(lattice,
                              position,
                              types,
@@ -1476,6 +1385,7 @@ static int get_symmetry_with_collinear_spin(int rotation[][3][3],
   }
   spg_free_dataset(dataset);
   dataset = NULL;
+#endif
 
   if ((symmetry = spn_get_collinear_operations(equivalent_atoms,
                                                sym_nonspin,
@@ -1871,43 +1781,34 @@ static int get_international(char symbol[11],
                              const double symprec,
                              const double angle_tolerance)
 {
-  Cell *cell;
-  Primitive *primitive;
-  Spacegroup spacegroup;
+  SpglibDataset *dataset;
+  int number;
 
-  cell = NULL;
-  primitive = NULL;
-  spacegroup.number = 0;
+  dataset = NULL;
 
-  if ((cell = cel_alloc_cell(num_atom)) == NULL) {
+  if ((dataset = get_dataset(lattice,
+                             position,
+                             types,
+                             num_atom,
+                             0,
+                             symprec,
+                             angle_tolerance)) == NULL) {
     goto err;
   }
 
-  cel_set_cell(cell, lattice, position, types);
-
-  if ((primitive = spa_get_spacegroup(&spacegroup,
-                                      cell,
-                                      0,
-                                      symprec,
-                                      angle_tolerance)) == NULL) {
-    cel_free_cell(cell);
-    cell = NULL;
-    goto err;
-  }
-
-  prm_free_primitive(primitive);
-  primitive = NULL;
-  cel_free_cell(cell);
-  cell = NULL;
-
-  if (spacegroup.number > 0) {
-    strcpy(symbol, spacegroup.international_short);
+  if (dataset->spacegroup_number > 0) {
+    number = dataset->spacegroup_number;
+    strcpy(symbol, dataset->international_symbol);
+    spg_free_dataset(dataset);
+    dataset = NULL;
   } else {
+    spg_free_dataset(dataset);
+    dataset = NULL;
     goto err;
   }
 
   spglib_error_code = SPGLIB_SUCCESS;
-  return spacegroup.number;
+  return number;
 
  err:
   spglib_error_code = SPGERR_SPACEGROUP_SEARCH_FAILED;
@@ -1922,48 +1823,40 @@ static int get_schoenflies(char symbol[7],
                            const double symprec,
                            const double angle_tolerance)
 {
-  Cell *cell;
-  Primitive *primitive;
-  Spacegroup spacegroup;
+  SpglibDataset *dataset;
+  SpglibSpacegroupType spgtype;
+  int number;
 
-  cell = NULL;
-  primitive = NULL;
-  spacegroup.number = 0;
+  dataset = NULL;
 
-  if ((cell = cel_alloc_cell(num_atom)) == NULL) {
+  if ((dataset = get_dataset(lattice,
+                             position,
+                             types,
+                             num_atom,
+                             0,
+                             symprec,
+                             angle_tolerance)) == NULL) {
     goto err;
   }
 
-  cel_set_cell(cell, lattice, position, types);
-
-  if ((primitive = spa_get_spacegroup(&spacegroup,
-                                      cell,
-                                      0,
-                                      symprec,
-                                      angle_tolerance)) == NULL) {
-    cel_free_cell(cell);
-    cell = NULL;
-    goto err;
-  }
-
-  prm_free_primitive(primitive);
-  primitive = NULL;
-  cel_free_cell(cell);
-  cell = NULL;
-
-  if (spacegroup.number > 0) {
-    strcpy(symbol, spacegroup.schoenflies);
+  if (dataset->spacegroup_number > 0) {
+    number = dataset->spacegroup_number;
+    spgtype = spg_get_spacegroup_type(dataset->hall_number);
+    strcpy(symbol, spgtype.schoenflies);
+    spg_free_dataset(dataset);
+    dataset = NULL;
   } else {
+    spg_free_dataset(dataset);
+    dataset = NULL;
     goto err;
   }
 
   spglib_error_code = SPGLIB_SUCCESS;
-  return spacegroup.number;
+  return number;
 
  err:
   spglib_error_code = SPGERR_SPACEGROUP_SEARCH_FAILED;
   return 0;
-
 }
 
 
